// serial.c:  For communicating with the motor controller
//
//  int  COM_init()		// Called at startup from set_ini().
//  void COMsendS(char *c_buf)  // Send a string 
//  int  COMgetch(void)		// Get a char from COM, or -1
//  int  COMgetPrompt(void)	// Wait for prompt char '>' (ctrlC breaks)
//  void COMflushIn(void)
//
//  3/13/96 pmt
//*************************************************************************/

#define USECOMM      // yes, we need the COMM API

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <assert.h>
#include <io.h>       /* */
#include "serial.h"


/* .ini section [COM_PORT] holds modifications of the following values...*/
int COM_port = 1;            // COM defaults to this value (0= dont use)
int COM_baud = 115200;
char COMname[30];

HANDLE hCom;		// Handle to opened COM device

//***********************************************
void PrintLastError(char *string1, char *string2)
{  // print out the LastError to console
      printf("%s: %s: Error # %d\n", string1, string2, GetLastError());
}

//**************************
int COMputchar(char ch)    // Send a character to the COM
{
DWORD Ndone;

// BOOL WriteFile(
//    HANDLE  hFile,	// handle to file to write to 
//    LPCVOID  lpBuffer,	// pointer to data to write to file 
//    DWORD  nNumberOfBytesToWrite,	// number of bytes to write 
//    LPDWORD  lpNumberOfBytesWritten,	// pointer to number of bytes written 
//    LPOVERLAPPED  lpOverlapped 	// addr. of structure needed for overlapped I/O  
//   );

    if (!WriteFile(hCom, &ch, (DWORD)1, &Ndone, NULL))
    {
		PrintLastError("WriteFile", COMname);
    }
    if (Ndone !=1) 
	{
		printf("Failed to write char to Com port.\n");		
		return(0);
	}
	return(1);
}


//**************************
void COMsendS(char *buf)    // Send a string to the COM
{
DWORD Ntodo, Ndone;
//BOOL WriteFile(
//    HANDLE  hFile,	// handle to file to write to 
//    LPCVOID  lpBuffer,	// pointer to data to write to file 
//    DWORD  nNumberOfBytesToWrite,	// number of bytes to write 
//    LPDWORD  lpNumberOfBytesWritten,	// pointer to number of bytes written 
//    LPOVERLAPPED  lpOverlapped 	// addr. of structure needed for overlapped I/O  
//   );

    Ntodo = strlen(buf);

    if (!WriteFile(hCom, buf, Ntodo, &Ndone, NULL))
    {
		PrintLastError("WriteFile", COMname);
    }
    if (Ntodo != Ndone) printf("Writefile(%s): todo=%d, done=%d!\n",COMname, Ntodo, Ndone);
}


/********************************/
int COMgetch(void)
{
char chars[8];
unsigned long count;

//BOOL ReadFile(   
//    HANDLE  hFile,	// handle of file to read 
//    LPVOID  lpBuffer,	// address of buffer that receives data  
//    DWORD  nNumberOfBytesToRead,	// number of bytes to read 
//    LPDWORD  lpNumberOfBytesRead,	// address of number of bytes read 
//    LPOVERLAPPED  lpOverlapped 	// address of structure for data 
//   );	
 
if (!ReadFile(hCom, chars, 1, &count, NULL ))
 {
     PrintLastError("ReadFile", "1 char");
 }
 if (count > 0) return((int) chars[0]);	// Return char or negative # if none
 else           return(-1);
}

/********************************/
int COMgetPrompt(void)		// Wait for prompt char '>' (ctrlC breaks??)
{
#define MAXWAIT 10		// Give up after about 1 sec.
#define MAXCHARS 100            // or 100 chars?
int c, t, i, ret;
char inbuf[MAXCHARS*3 +1];		// Buffer for incoming chars (nb hex chars take 3 spaces)

  i = 0;			// empty inbuf
  ret = FALSE;			// unless we get > char.

  for (t=0; t<MAXWAIT; t++) {	// Wait up to MAXWAIT ticks
    c = COMgetch();
    if ((c >= 0) && (i<MAXCHARS)) { // Got a character? then put it into buffer
      if (isprint(c)) inbuf[i++] = (char) c;	// alphas go as is
      else {
        sprintf(inbuf+i, "x%02x", c);		// Else as hex char x12
        i = strlen(inbuf);
      }
      inbuf[i] = 0;		// Terminate buffer
    }	// end of storing char
    if (c == '>') {
      ret = TRUE;		// Got prompt EXIT
      if (i==1) i = 0;		// Don't print out just prompt.
      break;
    }
    Sleep(10);			// arg is millisec in Win32
  } // next tick

  // Print any accumulated chars to command window
  if (i > 0) printf("COM:%s\n", inbuf);

  COMflushIn();		//Deal with occasional 2nd prompts!
  
  return(ret);
} // end COMgetPrompt()

/********************************/
void COMflushIn(void)
{
int c, i;

  i = 0;			// empty inbuf

    while ((c = COMgetch()) >= 0) { // For any chars in input
		putchar(c);
  } // next char


} // end COMflushIn()


/************************/
int COM_init()		// Called at startup
{
DCB dcb;
COMMTIMEOUTS cto;


  if ((COM_port <1) || (COM_port > 255)) {
	printf("COM port COM%d not initialized!\n", COM_port);
	COM_port = -1;
	return(FALSE);
  }


  // The following example opens a handle to COM1 and fills in a DCB structure 
  // with the current configuration. The DCB structure is then modified and 
  // used to reconfigure the device. 

  sprintf(COMname, "\\\\.\\COM%d", COM_port);

  hCom = CreateFile(COMname,
							  GENERIC_READ | GENERIC_WRITE,
							  0,    /* comm devices must be opened w/exclusive-access */
							  0,	/* no security attrs */
							  OPEN_EXISTING, /* comm devices must use OPEN_EXISTING */
							  0,    /* not overlapped I/O */
							  0		/* hTemplate must be NULL for comm devices */
					);

  if (hCom == INVALID_HANDLE_VALUE) {
      PrintLastError("CreateFile", COMname);
      return(FALSE);
  }

  /*
   * Omit the call to SetupComm to use the default queue sizes.
     pmt: ?? can't find info on default Q sizes ??.
   * Get the current configuration.
   */

  if (!GetCommState(hCom, &dcb)) {
      PrintLastError("GetCommState", COMname);
      return(FALSE);
  }

  /* Fill in the DCB: baud=19200, 8 data bits, no parity, 1 stop bit. */
  #ifdef NOTDEF
  typedef struct _DCB { // dcb 
    DWORD DCBlength;           // sizeof(DCB) 
    DWORD BaudRate;            // current baud rate 
    DWORD fBinary: 1;          // binary mode, no EOF check 
    DWORD fParity: 1;          // enable parity checking 
    DWORD fOutxCtsFlow:1;      // CTS output flow control 
    DWORD fOutxDsrFlow:1;      // DSR output flow control 
    DWORD fDtrControl:2;       // DTR flow control type 
    DWORD fDsrSensitivity:1;   // DSR sensitivity 
    DWORD fTXContinueOnXoff:1; // XOFF continues Tx 
    DWORD fOutX: 1;            // XON/XOFF out flow control 
    DWORD fInX: 1;             // XON/XOFF in flow control 
    DWORD fErrorChar: 1;       // enable error replacement 
    DWORD fNull: 1;            // enable null stripping 
    DWORD fRtsControl:2;       // RTS flow control 
    DWORD fAbortOnError:1;     // abort reads/writes on error 
    DWORD fDummy2:17;          // reserved 
    WORD wReserved;            // not currently used 
    WORD XonLim;               // transmit XON threshold 
    WORD XoffLim;              // transmit XOFF threshold 
    BYTE ByteSize;             // number of bits/byte, 4-8 
    BYTE Parity;               // 0-4=no,odd,even,mark,space 
    BYTE StopBits;             // 0,1,2 = 1, 1.5, 2 
    char XonChar;              // Tx and Rx XON character 
    char XoffChar;             // Tx and Rx XOFF character 
    char ErrorChar;            // error replacement character 
    char EofChar;              // end of input character 
    char EvtChar;              // received event character 
    WORD wReserved1;           // reserved; do not use 
} DCB; 
#endif

  dcb.BaudRate = COM_baud;
  dcb.ByteSize = 8;
  dcb.Parity = NOPARITY;
  dcb.StopBits = ONESTOPBIT;
  //dcb.StopBits = TWOSTOPBITS;
  dcb.fOutxCtsFlow = 0;      // CTS output flow control 
  dcb.fOutxDsrFlow = 0;      // DSR output flow control 
  dcb.fDsrSensitivity = 0;   // DSR sensitivity 
  dcb.fTXContinueOnXoff = 0; // XOFF continues Tx 
  dcb.fOutX = 0;             // XON/XOFF out flow control 
  dcb.fInX = 0;              // XON/XOFF in flow control 
  //dcb.fErrorChar: 1;       // enable error replacement 
  //dcb.fNull: 1;            // enable null stripping 
  dcb.fRtsControl = 0;       // RTS flow control 

  if (!SetCommState(hCom, &dcb)) {
      PrintLastError("SetCommState", COMname);
      return(FALSE);
  }

  //typedef struct _COMMTIMEOUTS {   // cto 
  //    DWORD ReadIntervalTimeout; 
  //    DWORD ReadTotalTimeoutMultiplier; 
  //    DWORD ReadTotalTimeoutConstant; 
  //    DWORD WriteTotalTimeoutMultiplier; 
  //    DWORD WriteTotalTimeoutConstant; 
  //} COMMTIMEOUTS,*LPCOMMTIMEOUTS; 
 
  // Fill in a timeout struct for immediate return on Reads
  cto.ReadIntervalTimeout = MAXDWORD; 
  cto.ReadTotalTimeoutMultiplier = 0; 
  cto.ReadTotalTimeoutConstant    = 0; 
  cto.WriteTotalTimeoutMultiplier = 1; 
  cto.WriteTotalTimeoutConstant = 2; 
  assert(SetCommTimeouts(hCom, &cto));

  return(TRUE);
} // end com_init()

//*****************************
int CloseCOM()
{
	return(CloseHandle(hCom));
}

